@@ -6,20 +6,22 @@ import (
 	"crypto/rsa"
 	"crypto/x509"
 	"encoding/pem"
+	"fmt"
 	"net/http"
 	"reflect"
-	"sort"
 	"strconv"
 	"testing"
 	"time"
 
 	"github.com/go-logr/logr"
+	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-github/v42/github"
 	"github.com/stretchr/testify/assert"
 	"golang.org/x/sync/errgroup"
 	"google.golang.org/protobuf/types/known/anypb"
 	"gopkg.in/h2non/gock.v1"
 
+	"github.com/trufflesecurity/trufflehog/v3/pkg/cache/memory"
 	"github.com/trufflesecurity/trufflehog/v3/pkg/context"
 	"github.com/trufflesecurity/trufflehog/v3/pkg/pb/credentialspb"
 	"github.com/trufflesecurity/trufflehog/v3/pkg/pb/sourcespb"
@@ -65,17 +67,23 @@ func TestAddReposByOrg(t *testing.T) {
 		Get("/orgs/super-secret-org/repos").
 		Reply(200).
 		JSON([]map[string]string{
-			{"clone_url": "super-secret-repo", "name": "super-secret-repo"},
-			{"clone_url": "super-secret-repo2", "full_name": "secret/super-secret-repo2"},
+			{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"},
+			{"clone_url": "https://github.com/super-secret-repo2.git", "full_name": "secret/super-secret-repo2"},
 		})
 
-	s := initTestSource(nil)
-	s.ignoreRepos = []string{"secret/super-*-repo2"}
+	s := initTestSource(&sourcespb.GitHub{
+		Credential: &sourcespb.GitHub_Token{
+			Token: "super secret token",
+		},
+		IncludeRepos: nil,
+		IgnoreRepos:  []string{"secret/super-*-repo2"},
+	})
 	// gock works here because github.NewClient is using the default HTTP Transport
-	err := s.addRepos(context.TODO(), "super-secret-org", s.getReposByOrg)
+	err := s.getReposByOrg(context.TODO(), "super-secret-org")
 	assert.Nil(t, err)
-	assert.Equal(t, 1, len(s.repos))
-	assert.Equal(t, []string{"super-secret-repo"}, s.repos)
+	assert.Equal(t, 1, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("super-secret-repo")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -86,21 +94,26 @@ func TestAddReposByOrg_IncludeRepos(t *testing.T) {
 		Get("/orgs/super-secret-org/repos").
 		Reply(200).
 		JSON([]map[string]string{
-			{"clone_url": "super-secret-repo", "full_name": "secret/super-secret-repo"},
-			{"clone_url": "super-secret-repo2", "full_name": "secret/super-secret-repo2"},
-			{"clone_url": "super-secret-repo2", "full_name": "secret/not-super-secret-repo"},
+			{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "secret/super-secret-repo"},
+			{"clone_url": "https://github.com/super-secret-repo2.git", "full_name": "secret/super-secret-repo2"},
+			{"clone_url": "https://github.com/super-secret-repo2.git", "full_name": "secret/not-super-secret-repo"},
 		})
 
-	src := &sourcespb.GitHub{
+	s := initTestSource(&sourcespb.GitHub{
+		Credential: &sourcespb.GitHub_Token{
+			Token: "super secret token",
+		},
+		IncludeRepos:  []string{"secret/super*"},
 		Organizations: []string{"super-secret-org"},
-	}
-	s := initTestSource(src)
-	s.includeRepos = []string{"secret/super*"}
+	})
 	// gock works here because github.NewClient is using the default HTTP Transport
-	err := s.addRepos(context.TODO(), "super-secret-org", s.getReposByOrg)
+	err := s.getReposByOrg(context.TODO(), "super-secret-org")
 	assert.Nil(t, err)
-	assert.Equal(t, 2, len(s.repos))
-	assert.Equal(t, []string{"super-secret-repo", "super-secret-repo2"}, s.repos)
+	assert.Equal(t, 2, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("secret/super-secret-repo")
+	assert.True(t, ok)
+	ok = s.filteredRepoCache.Exists("secret/super-secret-repo2")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -111,16 +124,21 @@ func TestAddReposByUser(t *testing.T) {
 		Get("/users/super-secret-user/repos").
 		Reply(200).
 		JSON([]map[string]string{
-			{"clone_url": "super-secret-repo", "name": "super-secret-repo"},
-			{"clone_url": "super-secret-repo2", "full_name": "secret/super-secret-repo2"},
+			{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"},
+			{"clone_url": "https://github.com/super-secret-repo2.git", "full_name": "secret/super-secret-repo2"},
 		})
 
-	s := initTestSource(nil)
-	s.ignoreRepos = []string{"secret/super-secret-repo2"}
-	err := s.addRepos(context.TODO(), "super-secret-user", s.getReposByUser)
+	s := initTestSource(&sourcespb.GitHub{
+		Credential: &sourcespb.GitHub_Token{
+			Token: "super secret token",
+		},
+		IgnoreRepos: []string{"secret/super-secret-repo2"},
+	})
+	err := s.getReposByUser(context.TODO(), "super-secret-user")
 	assert.Nil(t, err)
-	assert.Equal(t, 1, len(s.repos))
-	assert.Equal(t, []string{"super-secret-repo"}, s.repos)
+	assert.Equal(t, 1, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("super-secret-repo")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -130,13 +148,36 @@ func TestAddGistsByUser(t *testing.T) {
 	gock.New("https://api.github.com").
 		Get("/users/super-secret-user/gists").
 		Reply(200).
-		JSON([]map[string]string{{"git_pull_url": "super-secret-gist"}})
+		JSON([]map[string]string{{"git_pull_url": "https://githug.com/super-secret-gist.git", "id": "super-secret-gist"}})
 
 	s := initTestSource(nil)
-	err := s.addGistsByUser(context.TODO(), "super-secret-user")
+	err := s.addUserGistsToCache(context.TODO(), "super-secret-user")
 	assert.Nil(t, err)
-	assert.Equal(t, 1, len(s.repos))
-	assert.Equal(t, []string{"super-secret-gist"}, s.repos)
+	assert.Equal(t, 1, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("super-secret-gist")
+	assert.True(t, ok)
+	assert.True(t, gock.IsDone())
+}
+
+func TestAddMembersByOrg(t *testing.T) {
+	defer gock.Off()
+
+	gock.New("https://api.github.com").
+		Get("/orgs/org1/members").
+		Reply(200).
+		JSON([]map[string]string{
+			{"login": "testman1"},
+			{"login": "testman2"},
+		})
+
+	s := initTestSource(nil)
+	err := s.addMembersByOrg(context.TODO(), "org1")
+	assert.Nil(t, err)
+	assert.Equal(t, 2, len(s.memberCache))
+	_, ok := s.memberCache["testman1"]
+	assert.True(t, ok)
+	_, ok = s.memberCache["testman2"]
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -161,8 +202,13 @@ func TestAddMembersByApp(t *testing.T) {
 	s := initTestSource(nil)
 	err := s.addMembersByApp(context.TODO(), github.NewClient(nil))
 	assert.Nil(t, err)
-	assert.Equal(t, 3, len(s.members))
-	assert.Equal(t, []string{"ssm1", "ssm2", "ssm3"}, s.members)
+	assert.Equal(t, 3, len(s.memberCache))
+	_, ok := s.memberCache["ssm1"]
+	assert.True(t, ok)
+	_, ok = s.memberCache["ssm2"]
+	assert.True(t, ok)
+	_, ok = s.memberCache["ssm3"]
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -174,16 +220,19 @@ func TestAddReposByApp(t *testing.T) {
 		Reply(200).
 		JSON(map[string]interface{}{
 			"repositories": []map[string]string{
-				{"clone_url": "ssr1"},
-				{"clone_url": "ssr2"},
+				{"clone_url": "https://github/ssr1.git", "full_name": "ssr1"},
+				{"clone_url": "https://github/ssr2.git", "full_name": "ssr2"},
 			},
 		})
 
 	s := initTestSource(nil)
 	err := s.addReposByApp(context.TODO())
 	assert.Nil(t, err)
-	assert.Equal(t, 2, len(s.repos))
-	assert.Equal(t, []string{"ssr1", "ssr2"}, s.repos)
+	assert.Equal(t, 2, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("ssr1")
+	assert.True(t, ok)
+	ok = s.filteredRepoCache.Exists("ssr2")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -201,8 +250,9 @@ func TestAddOrgsByUser(t *testing.T) {
 
 	s := initTestSource(nil)
 	s.addOrgsByUser(context.TODO(), "super-secret-user")
-	assert.Equal(t, 1, len(s.orgs))
-	assert.Equal(t, []string{"sso2"}, s.orgs)
+	assert.Equal(t, 1, s.orgsCache.Count())
+	ok := s.orgsCache.Exists("sso2")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -213,30 +263,15 @@ func TestNormalizeRepos(t *testing.T) {
 		name     string
 		setup    func()
 		repos    []string
-		expected []string
+		expected map[string]struct{}
+		wantErr  bool
 	}{
 		{
-			name:     "repo url",
-			setup:    func() {},
-			repos:    []string{"https://github.com/super-secret-user/super-secret-repo"},
-			expected: []string{"https://github.com/super-secret-user/super-secret-repo.git"},
-		},
-		{
-			name: "username with gists",
-			setup: func() {
-				gock.New("https://api.github.com").
-					Get("/users/super-secret-user/gists").
-					Reply(200).
-					JSON([]map[string]string{{"git_pull_url": "https://github.com/super-secret-user/super-secret-gist.git"}})
-				gock.New("https://api.github.com").
-					Get("/users/super-secret-user/repos").
-					Reply(200).
-					JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-user/super-secret-repo.git"}})
-			},
-			repos: []string{"super-secret-user"},
-			expected: []string{
-				"https://github.com/super-secret-user/super-secret-repo.git",
-				"https://github.com/super-secret-user/super-secret-gist.git",
+			name:  "repo url",
+			setup: func() {},
+			repos: []string{"https://github.com/super-secret-user/super-secret-repo"},
+			expected: map[string]struct{}{
+				"https://github.com/super-secret-user/super-secret-repo.git": {},
 			},
 		},
 		{
@@ -250,13 +285,15 @@ func TestNormalizeRepos(t *testing.T) {
 					Reply(404)
 			},
 			repos:    []string{"not-found"},
-			expected: []string{},
+			expected: map[string]struct{}{},
+			wantErr:  true,
 		},
 		{
 			name:     "unexpected format",
 			setup:    func() {},
 			repos:    []string{"/foo/"},
-			expected: []string{},
+			expected: map[string]struct{}{},
+			wantErr:  true,
 		},
 	}
 
@@ -265,16 +302,25 @@ func TestNormalizeRepos(t *testing.T) {
 			defer gock.Off()
 			tt.setup()
 			s := initTestSource(nil)
-			s.repos = tt.repos
 
-			s.normalizeRepos(context.TODO())
-			assert.Equal(t, len(tt.expected), len(s.repos))
-			// sort and compare
-			sort.Slice(tt.expected, func(i, j int) bool { return tt.expected[i] < tt.expected[j] })
-			sort.Slice(s.repos, func(i, j int) bool { return s.repos[i] < s.repos[j] })
-			assert.Equal(t, tt.expected, s.repos)
+			got, err := s.normalizeRepo(tt.repos[0])
+			if (err != nil) != tt.wantErr {
+				t.Errorf("normalizeRepo() error = %v, wantErr %v", err, tt.wantErr)
+				return
+			}
+			if got != "" {
+				for k := range tt.expected {
+					assert.Equal(t, got, k)
+				}
+			}
+			res := make(map[string]struct{}, s.filteredRepoCache.Count())
+			for _, v := range s.filteredRepoCache.Keys() {
+				res[v] = struct{}{}
+			}
 
-			assert.True(t, gock.IsDone())
+			if got == "" && !cmp.Equal(res, tt.expected) {
+				t.Errorf("normalizeRepo() got = %v, want %v", s.repos, tt.expected)
+			}
 		})
 	}
 }
@@ -296,13 +342,15 @@ func TestEnumerateUnauthenticated(t *testing.T) {
 	gock.New("https://api.github.com").
 		Get("/orgs/super-secret-org/repos").
 		Reply(200).
-		JSON([]map[string]string{{"clone_url": "super-secret-repo"}})
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"}})
 
 	s := initTestSource(nil)
-	s.orgs = []string{"super-secret-org"}
+	s.orgsCache = memory.New()
+	s.orgsCache.Set("super-secret-org", "super-secret-org")
 	s.enumerateUnauthenticated(context.TODO())
-	assert.Equal(t, 1, len(s.repos))
-	assert.Equal(t, []string{"super-secret-repo"}, s.repos)
+	assert.Equal(t, 1, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("super-secret-repo")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
@@ -317,21 +365,157 @@ func TestEnumerateWithToken(t *testing.T) {
 	gock.New("https://api.github.com").
 		Get("/users/super-secret-user/repos").
 		Reply(200).
-		JSON([]map[string]string{{"clone_url": "super-secret-repo"}})
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"}})
+
+	gock.New("https://api.github.com").
+		Get("/user/orgs").
+		MatchParam("per_page", "100").
+		Reply(200).
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"}})
 
 	gock.New("https://api.github.com").
 		Get("/users/super-secret-user/gists").
 		Reply(200).
-		JSON([]map[string]string{{"clone_url": ""}})
+		JSON([]map[string]string{{"git_pull_url": "https://github.com/super-secret-gist.git", "id": "super-secret-gist"}})
 
 	s := initTestSource(nil)
 	err := s.enumerateWithToken(context.TODO(), "https://api.github.com", "token")
 	assert.Nil(t, err)
-	assert.Equal(t, 2, len(s.repos))
-	assert.Equal(t, []string{"super-secret-repo", ""}, s.repos)
+	assert.Equal(t, 2, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("super-secret-repo")
+	assert.True(t, ok)
+	ok = s.filteredRepoCache.Exists("super-secret-gist")
+	assert.True(t, ok)
 	assert.True(t, gock.IsDone())
 }
 
+func BenchmarkEnumerateWithToken(b *testing.B) {
+	defer gock.Off()
+
+	gock.New("https://api.github.com").
+		Get("/user").
+		Reply(200).
+		JSON(map[string]string{"login": "super-secret-user"})
+
+	gock.New("https://api.github.com").
+		Get("/users/super-secret-user/repos").
+		Reply(200).
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git"}})
+
+	gock.New("https://api.github.com").
+		Get("/user/orgs").
+		MatchParam("per_page", "100").
+		Reply(200).
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git"}})
+
+	gock.New("https://api.github.com").
+		Get("/users/super-secret-user/gists").
+		Reply(200).
+		JSON([]map[string]string{{"git_pull_url": "https://github.com/super-secret-gist.git"}})
+
+	s := initTestSource(nil)
+
+	b.ResetTimer()
+	for i := 0; i < b.N; i++ {
+		_ = s.enumerateWithToken(context.TODO(), "https://api.github.com", "token")
+	}
+}
+
+func TestEnumerate(t *testing.T) {
+	defer gock.Off()
+
+	gock.New("https://api.github.com").
+		Get("/user").
+		Reply(200).
+		JSON(map[string]string{"login": "super-secret-user"})
+
+	gock.New("https://api.github.com").
+		Get("/users/super-secret-user/repos").
+		Reply(200).
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"}})
+
+	gock.New("https://api.github.com").
+		Get("/user/orgs").
+		MatchParam("per_page", "100").
+		Reply(200).
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git", "full_name": "super-secret-repo"}})
+
+	gock.New("https://api.github.com").
+		Get("/users/super-secret-user/gists").
+		Reply(200).
+		JSON([]map[string]string{{"git_pull_url": "https://github.com/super-secret-gist.git", "id": "super-secret-gist"}})
+
+	s := initTestSource(&sourcespb.GitHub{
+		Credential: &sourcespb.GitHub_Token{
+			Token: "super secret token",
+		},
+	})
+
+	_, err := s.enumerate(context.TODO(), "https://api.github.com")
+	assert.Nil(t, err)
+	assert.Equal(t, 2, s.filteredRepoCache.Count())
+	ok := s.filteredRepoCache.Exists("super-secret-repo")
+	assert.True(t, ok)
+	ok = s.filteredRepoCache.Exists("super-secret-gist")
+	assert.True(t, ok)
+	assert.True(t, gock.IsDone())
+}
+
+func setupMocks(b *testing.B) {
+	b.Helper()
+
+	gock.New("https://api.github.com").
+		Get("/user").
+		Reply(200).
+		JSON(map[string]string{"login": "super-secret-user"})
+
+	gock.New("https://api.github.com").
+		Get("/users/super-secret-user/repos").
+		Reply(200).
+		JSON(mockRepos())
+
+	gock.New("https://api.github.com").
+		Get("/user/orgs").
+		MatchParam("per_page", "100").
+		Reply(200).
+		JSON([]map[string]string{{"clone_url": "https://github.com/super-secret-repo.git"}})
+
+	gock.New("https://api.github.com").
+		Get("/users/super-secret-user/gists").
+		Reply(200).
+		JSON(mockGists())
+}
+
+func mockRepos() []map[string]string {
+	res := make([]map[string]string, 0, 10000)
+	for i := 0; i < 10000; i++ {
+		res = append(res, map[string]string{"clone_url": fmt.Sprintf("https://githu/super-secret-repo-%d.git", i)})
+	}
+	return res
+}
+
+func mockGists() []map[string]string {
+	res := make([]map[string]string, 0, 100)
+	for i := 0; i < 100; i++ {
+		res = append(res, map[string]string{"git_pull_url": fmt.Sprintf("https://githu/super-secret-gist-%d.git", i)})
+	}
+	return res
+}
+
+func BenchmarkEnumerate(b *testing.B) {
+	for i := 0; i < b.N; i++ {
+		s := initTestSource(&sourcespb.GitHub{
+			Credential: &sourcespb.GitHub_Token{
+				Token: "super secret token",
+			},
+		})
+		setupMocks(b)
+
+		b.StartTimer()
+		_, _ = s.enumerate(context.TODO(), "https://api.github.com")
+	}
+}
+
 func TestEnumerateWithToken_IncludeRepos(t *testing.T) {
 	defer gock.Off()
 
@@ -498,9 +682,10 @@ func Test_scan_SetProgressComplete(t *testing.T) {
 			wantComplete: true,
 		},
 		{
-			name:    "one valid repo",
-			repos:   []string{"a"},
-			wantErr: true,
+			name:         "one valid repo",
+			repos:        []string{"a"},
+			wantComplete: true,
+			wantErr:      true,
 		},
 	}
 
